from .orthanc_cloner import OrthancCloner, ClonerMode
from .orthanc_folder_importer import *
from .orthanc_forwarder import *
from .orthanc_monitor import OrthancMonitor
from .orthanc_test_db_populator import OrthancTestDbPopulator
from .pacs_migrator import PacsMigrator
from .orthanc_comparator import OrthancComparator
from .orthanc_cleaner import OrthancCleaner
from .orthanc_replicator import OrthancReplicator
from .ids_migrator import IdsMigrator

from .hl7Lib import *
from .helpers import *

# Set default logging handler to avoid "No handler found" warnings.
import logging
from logging import NullHandler

logger = logging.getLogger(__name__)
logger.addHandler(NullHandler())




import inquirer
from pathlib import Path
import logging
import argparse
from orthanc_api_client import OrthancApiClient

logger = logging.getLogger(__name__)

'''
This script is interactive!
So, run it from the console, follow instructions and enjoy...
'''

class OrthancUploader:
    def __init__(self,
                 api_client: OrthancApiClient,
                 path: str
                 ):
        self._api_client = api_client
        self._path = path

    def get_folder(self, current_path = Path('.')):
        """ Print a numbered list of the subfolders in the working directory
        (i.e. the directory the script is run from), and returns the directory
        the user chooses.
        """
        answers = {'studies_path': 'None'}

        while answers['studies_path'] != '.':
            print("Browse to the folder you want to upload to Orthanc, then select the . to validate.")
            questions = [
                inquirer.List(
                    "studies_path",
                    message="Curent folder: " + str(current_path.absolute()),
                    choices=['.'] + ['..'] + [str(d) for d in current_path.iterdir() if d.is_dir()]
                )
            ]
            answers = inquirer.prompt(questions)
            if answers['studies_path'] == '..':
                current_path = current_path.absolute().parent
            elif answers['studies_path'] != '.':
                current_path = Path(answers['studies_path'])
        return str(current_path.absolute())

    def execute(self):

        # let the user choose the folder to upload
        path = self.get_folder(Path(self._path))

        # let's get the existing labels
        labels_list = orthanc_client.get_all_labels()
        orthanc_labels_chosen_list = []

        # let the user choose the labels to apply among the existing ones
        if len(labels_list)>0:
            questions_labels = [
                inquirer.Checkbox(
                    "orthanc_labels_chosen_list",
                    message="Choose the existing labels to apply",
                    choices=labels_list,
                )
            ]

            answers = inquirer.prompt(questions_labels)
            orthanc_labels_chosen_list = answers['orthanc_labels_chosen_list']

        # let the user manually add extra labels
        answers = {'orthanc_extra_label': 'None'}
        while answers['orthanc_extra_label'] != '':
            print(f"Current labels list: {orthanc_labels_chosen_list}")
            questions_labels_manual = [
                inquirer.Text(
                    "orthanc_extra_label",
                    message="Type an extra label to apply, leave it empty and press ENTER if any"
                ),
            ]

            answers = inquirer.prompt(questions_labels_manual)
            orthanc_extra_label = answers['orthanc_extra_label']
            if orthanc_extra_label != "":
                orthanc_labels_chosen_list.append(orthanc_extra_label)

        print("-----------------------------------------------------------------------------------")
        print(f"Folder to import: {path}")
        print(f"Labels to apply: {orthanc_labels_chosen_list}")
        questions_continue = [
            inquirer.Confirm("continue", message="Ready to start?", default=True)
        ]

        answers = inquirer.prompt(questions_continue)
        if answers['continue'] == False:
            print("Aborted!")
            exit(-1)

        logger.info(f"starting import of folder {path} ...")
        dicom_ids_set, orthanc_ids_set, rejected_files_list = orthanc_client.upload_folder_return_details(path)

        logger.info(f"starting labeling...")
        for id in orthanc_ids_set:
            orthanc_client.studies.add_labels(orthanc_id=id, labels=orthanc_labels_chosen_list)

        if len(rejected_files_list)>0:
            logger.info("End of upload, here are files in error:")
            for rejected_file in rejected_files_list:
                logger.info(f"{rejected_file[0]} --- {rejected_file[1]}")
        logger.info("End of upload, no errors :-)")


if __name__ == '__main__':
    level = logging.INFO

    logging.basicConfig(level=level, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    parser = argparse.ArgumentParser(description="Upload studies from a selected folder and recursively in all subfolders to an Orthanc and apply labels.")
    parser.add_argument('--url', help="The url of the Orthanc to upload studies to.", default="http://localhost:8042")
    parser.add_argument('--user', help="The user of the Orthanc to upload studies to.", default="demo")
    parser.add_argument('--password', help="The password of the Orthanc to upload studies to.", default="demo")
    parser.add_argument('--start_path', help="The path used as start path for the selection of the folder containing studies to upload. Default value: script execution path", default=".")

    args = parser.parse_args()

    orthanc_url = args.url
    orthanc_user = args.user
    orthanc_password = args.password
    start_path = args.start_path

    orthanc_client = OrthancApiClient(orthanc_url, user=orthanc_user, pwd=orthanc_password)

    uploader = OrthancUploader(api_client=orthanc_client, path=start_path)

    uploader.execute()



import argparse
import threading
import time

import pika
import logging
import os

from orthanc_api_client import OrthancApiClient, ResourceType, JobStatus, ResourceNotFound

logger = logging.getLogger(__name__)

class OrthancReplicator:
    '''
    ## Goal
    The Replicator allows to mirror an Orthanc (source) to another (destination):
    - Every instance received in the first Orthanc has to be forwarded to the second one;
    - Every instance deleted from the first Orthanc has to be deleted from the second one.

    ## Challenge
    The challenge comes from the lack of "DeletedInstance" change type in the changes returned by the
    "/changes" API route of Orthanc.

    ## Trick
    There is a lua event called "OnDeletedInstance". So, a lua script configured in the Orthanc source will
    push the orthanc id of every deleted instance to a RabbitMQ broker.
    This lua script will do the same for every instance stored (in a dedicated queue).
    Here comes the Replicator:
    It will consume these RabbitMQ messages to delete ("to-delete-queue") the instances from the destination
    or to forward ("to-forward-queue") the instances from the source to the destination.

    ## How it works
    ### On the source side
    There is a lua script to configure (see dedicated sample TODO: add url).
    Basically, this lua script should push messages to the broker (rabbitmq) following these rules:
    - vhost: "/" (default one)
    - exchange: "orthanc-exchange"
    - queue for instances to delete: "to-delete-queue"
    - queue for instances to forward: "to-forward-queue"

    Side note: these hardcoded names could be made configurable in a further version...

    ### Broker
    RabbitMQ has to be up and running.

    ### Replicator
    The replicator needs to connect to both Orthanc (source and destination) and to RabbitMQ.
    So these information should be provided:
    source_url: Orthanc source url
    source_user: Orthanc source user name
    source_pwd: Orthanc source password
    dest_url: Orthanc destination url
    dest_user: Orthanc destination user name
    dest_pwd: Orthanc destination password
    broker_url: Broker url
    broker_user: Broker user name
    broker_pwd: Broker password
    broker_port: Broker port nr
    '''

    def __init__(self,
                 source: OrthancApiClient,
                 destination: OrthancApiClient,
                 broker_params: pika.ConnectionParameters
        ):

        self._source = source
        self._destination = destination
        self._broker_params = broker_params
        self._consuming_thread = None

    def to_delete_callback(self, channel, method, properties, body):
        orthanc_id = body.decode('utf8')
        try:
            self._destination.instances.delete(orthanc_id)
            channel.basic_ack(delivery_tag=method.delivery_tag)
            logger.debug(f"Deleted instance from destination ({orthanc_id}).")
            return

        except ResourceNotFound as ex:
            # The instance may have been deleted (by a user, a script,...) from the destination between the moment
            # it has been deleted from the source (and the id pushed into rabbitmq)
            # and the moment we try to handle it. So, the destination will return a 404 http error.
            # In this case, simply log the error and ack the message (don't raise the Exception)
            logger.info(f"Unable (404) to delete instance from destination ({orthanc_id}), probably already deleted...")
            channel.basic_ack(delivery_tag=method.delivery_tag)

        except Exception as ex:
            logger.warning(f"Unable to delete instance from destination ({orthanc_id}), requeueing...")
            channel.basic_nack(delivery_tag=method.delivery_tag, requeue=False)

    def to_forward_callback(self, channel, method, properties, body):
        orthanc_id = body.decode('utf8')
        try:
            dicom = self._source.instances.get_file(orthanc_id)

        except ResourceNotFound as ex:
            # The instance may have been deleted (by a user, a script,...) from the source between the moment
            # it has been received in the source (and the id pushed into rabbitmq)
            # and the moment we try to handle it. So, the source will return a 404 http error.
            # In this case, simply log the error and ack the message (don't raise the Exception)
            logger.warning(f"Unable (404) to get instance instance from source ({orthanc_id}), probably already deleted...")
            channel.basic_ack(delivery_tag=method.delivery_tag)
            return
        except Exception as ex:
            logger.warning(f"Unable to get instance from source ({orthanc_id}), requeueing...")
            channel.basic_nack(delivery_tag=method.delivery_tag, requeue=False)
            return

        try:
            self._destination.upload(dicom)

            channel.basic_ack(delivery_tag=method.delivery_tag)
            logger.debug(f"Forwarded instance to destination ({orthanc_id}).")
            return

        except Exception as ex:
            channel.basic_nack(delivery_tag=method.delivery_tag, requeue=False)
            logger.warning(f"Unable to upload instance to destination ({orthanc_id}), requeueing...")
            return

    def wait_orthanc_started(self):
        retry = 0
        while not self._source.is_alive():
            logger.info("Waiting to connect to Orthanc source")
            retry += 1
            if retry == 5:
                logger.error("Could not connect to Orthanc at startup")
                raise Exception("Could not connect to Orthanc at startup")
            time.sleep(1)

        retry = 0
        while not self._destination.is_alive():
            logger.info("Waiting to connect to Orthanc destination")
            retry += 1
            if retry == 5:
                logger.error("Could not connect to Orthanc at startup")
                raise Exception("Could not connect to Orthanc at startup")
            time.sleep(1)


    def _consume(self):
        logger.info("----- Initializing Orthanc Replicator...")

        # TODO: allow multithreading?
        # see https://blog.boot.dev/python/using-concurrent-subscribers-rabbitmq-in-python-pika/

        self.wait_orthanc_started()

        # initialize connection to rabbitmq
        connection = pika.BlockingConnection(self._broker_params)
        channel = connection.channel()

        # These steps should have been done in the lua, but they are idempotent
        channel.exchange_declare(exchange="orthanc-exchange", exchange_type="direct", durable=True)

        # "Main" queues
        channel.queue_declare(queue='to-forward-queue',
                              durable=True,
                              arguments={
                                  'x-dead-letter-exchange': 'orthanc-exchange',
                                  'x-dead-letter-routing-key': 'standby-forward-queue'
                              })
        channel.queue_declare(queue='to-delete-queue', durable=True,
                              arguments={
                                  'x-dead-letter-exchange': 'orthanc-exchange',
                                  'x-dead-letter-routing-key': 'standby-delete-queue'
                              })

        # "Standby" queues (messages are waiting there is they were nacked)
        channel.queue_declare(queue='standby-delete-queue', durable=True,
                              arguments={
                                  'x-message-ttl': 10000,
                                  'x-dead-letter-exchange': 'orthanc-exchange',
                                  'x-dead-letter-routing-key': 'to-delete-queue'
                              })
        channel.queue_declare(queue='standby-forward-queue', durable=True,
                              arguments={
                                  'x-message-ttl': 10000,
                                  'x-dead-letter-exchange': 'orthanc-exchange',
                                  'x-dead-letter-routing-key': 'to-forward-queue'
                              })

        channel.queue_bind(exchange="orthanc-exchange", queue="to-forward-queue", routing_key="to-forward-queue")
        channel.queue_bind(exchange="orthanc-exchange", queue="to-delete-queue", routing_key="to-delete-queue")
        channel.queue_bind(exchange="orthanc-exchange", queue="standby-delete-queue", routing_key="standby-delete-queue")
        channel.queue_bind(exchange="orthanc-exchange", queue="standby-forward-queue", routing_key="standby-forward-queue")

        channel.basic_consume(queue='to-forward-queue', on_message_callback=self.to_forward_callback)
        channel.basic_consume(queue='to-delete-queue', on_message_callback=self.to_delete_callback)

        # we declare a "stop-queue" which allows to gracefully stop the connection with rabbitmq
        channel.queue_declare(queue='stop-queue')
        channel.queue_bind(exchange="orthanc-exchange", queue="stop-queue", routing_key="stop-queue")
        channel.basic_consume(queue='stop-queue', on_message_callback=self.stop_callback, auto_ack=True)

        logger.info("Broker connection configured, waiting for messages...")
        channel.start_consuming() # this never ends

        channel.stop_consuming()
        connection.close()

        logger.info("Connection closed -----")

    def stop_callback(self, channel, method, properties, body):
        channel.stop_consuming()
        channel.close()

        logger.info("Broker connection stop requested...")

    def stop(self):
        logger.info("Stopping Replicator...")

        connection = pika.BlockingConnection(self._broker_params)
        channel = connection.channel()

        channel.basic_publish(exchange='orthanc-exchange', routing_key='stop-queue', body="stop")

        connection.close()

    def execute(self):
        self._consuming_thread = threading.Thread(target=self._consume)
        self._consuming_thread.start()

# example:
# python orthanc_tools/orthanc_replicator.py --source_url=http://localhost:8042 --dest_url=http://localhost:8044 --broker_url=http://localhost


if __name__ == '__main__':
    level = logging.INFO

    if os.environ.get('VERBOSE_ENABLED'):
        level = logging.DEBUG

    logging.basicConfig(level=level, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    parser = argparse.ArgumentParser(description='Replicates the content of an Orthanc into another Orthanc')
    parser.add_argument('--source_url', type=str, default=None, help='Orthanc source url')
    parser.add_argument('--source_user', type=str, default=None, help='Orthanc source user name')
    parser.add_argument('--source_pwd', type=str, default=None, help='Orthanc source password')
    parser.add_argument('--dest_url', type=str, default=None, help='Orthanc destination url')
    parser.add_argument('--dest_user', type=str, default=None, help='Orthanc destination user name')
    parser.add_argument('--dest_pwd', type=str, default=None, help='Orthanc destination password')
    parser.add_argument('--broker_url', type=str, default='broker', help='Broker url')
    parser.add_argument('--broker_user', type=str, default='rabbit', help='Broker user name')
    parser.add_argument('--broker_pwd', type=str, default='123456', help='Broker password')
    parser.add_argument('--broker_port', type=int, default=5672, help='Broker port nr')
    #parser.add_argument('--worker_threads_count', type=int, default=1, help='Number of worker threads')

    args = parser.parse_args()

    source_url = os.environ.get("SOURCE_URL", args.source_url)
    source_user = os.environ.get("SOURCE_USER", args.source_user)
    source_pwd = os.environ.get("SOURCE_PWD", args.source_pwd)
    
    dest_url = os.environ.get("DEST_URL", args.dest_url)
    dest_user = os.environ.get("DEST_USER", args.dest_user)
    dest_pwd = os.environ.get("DEST_PWD", args.dest_pwd)
    
    broker_url = os.environ.get("BROKER_URL", args.broker_url)
    broker_user = os.environ.get("BROKER_USER", args.broker_user)
    broker_pwd = os.environ.get("BROKER_PWD", args.broker_pwd)
    broker_port = int(os.environ.get("BROKER_PORT", args.broker_port))
    
    #worker_threads_count = int(os.environ.get("WORKER_THREADS_COUNT", str(args.worker_threads_count)))

    broker_connection_parameters = pika.ConnectionParameters(broker_url, broker_port, credentials=pika.PlainCredentials(broker_user, broker_pwd))

    replicator = OrthancReplicator(
        source=OrthancApiClient(source_url, user=source_user, pwd=source_pwd),
        destination=OrthancApiClient(dest_url, user=dest_user, pwd=dest_pwd),
        broker_params=broker_connection_parameters
    )

    replicator.execute()






import os
import time
import logging
logger = logging.getLogger(__name__)


class Scheduler:
    """
    Helper class to run a task only at certain time of the day
    """

    def __init__(self, night_start_hour: int = 19, night_end_hour: int = 7, run_only_at_night_and_weekend: bool = False):
        self._night_start_hour = night_start_hour
        self._night_end_hour = night_end_hour
        self._run_only_at_night_and_weekend = run_only_at_night_and_weekend

    def wait_right_time_to_run(self):
        if self._run_only_at_night_and_weekend:
            is_right_time = False
            while not is_right_time:
                now = time.localtime()
                if now.tm_wday <= 4 and self._night_end_hour <= now.tm_hour < self._night_start_hour:
                    is_right_time = False
                    logger.info("waiting 10 min for the night or week-end to come")
                    time.sleep(600)
                else:
                    is_right_time = True


    @classmethod
    def add_parser_arguments(cls, parser):
        parser.add_argument('--run_only_at_night_and_weekend', default=False, action='store_true', help='enables script only at night')
        parser.add_argument('--night_start_hour', type=int, default=19, help='Night start hour')
        parser.add_argument('--night_end_hour', type=int, default=6, help='Night start hour')

    @classmethod
    def create_from_args_and_env_var(cls, args):
        night_start_hour = int(os.environ.get("NIGHT_START_HOUR", str(args.night_start_hour)))
        night_end_hour = int(os.environ.get("NIGHT_END_HOUR", str(args.night_end_hour)))
        if os.environ.get("RUN_ONLY_AT_NIGHT_AND_WEEKEND", None) is not None:
            run_only_at_night_and_weekend = os.environ.get("RUN_ONLY_AT_NIGHT_AND_WEEKEND") == "true"
        else:
            run_only_at_night_and_weekend = args.run_only_at_night_and_weekend

        return Scheduler(
            night_start_hour=night_start_hour,
            night_end_hour=night_end_hour,
            run_only_at_night_and_weekend=run_only_at_night_and_weekend
        )



import time
import subprocess
from orthanc_api_client import OrthancApiClient, helpers
import pathlib
import logging
import unittest
import pika

from orthanc_tools import OrthancReplicator

here = pathlib.Path(__file__).parent.resolve()

logger = logging.getLogger('orthanc_tools')
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)


class TestOrthancReplicator(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        subprocess.run(["docker", "compose", "down", "-v"], cwd=here/"docker-setup-replicator")
        subprocess.run(["docker", "compose", "up", "-d"], cwd=here/"docker-setup-replicator")

        cls.oa = OrthancApiClient('http://localhost:10042', user='test', pwd='test')
        cls.oa.wait_started()
        cls.ob = OrthancApiClient('http://localhost:10043', user='test', pwd='test')
        cls.ob.wait_started()

    @classmethod
    def tearDownClass(cls):
        subprocess.run(["docker", "compose", "down", "-v"], cwd=here/"docker-setup-replicator")

    def get_rabbitmq_connection_params(self):
        broker_connection_parameters = pika.ConnectionParameters(
            "localhost", 5672,
            credentials=pika.PlainCredentials("rabbit", "123456"),
            connection_attempts=3,
            heartbeat=300,
            socket_timeout=None,
            stack_timeout=None,
            blocked_connection_timeout=None
        )
        return broker_connection_parameters

    def get_queue_length(self, queue: str, standby: bool):
        pika_conn_params = self.get_rabbitmq_connection_params()
        connection = pika.BlockingConnection(pika_conn_params)
        channel = connection.channel()

        # only way to interact with a queue: declare it again
        # this is idempotent and the existing messages are not affected
        if standby:
            queue_name = f"standby-{queue}-queue"
            routing_key = f"to-{queue}-queue"
            arguments = {
                'x-message-ttl': 10000,
                'x-dead-letter-exchange': 'orthanc-exchange',
                'x-dead-letter-routing-key': routing_key
            }
        else:
            queue_name = f"to-{queue}-queue"
            routing_key = f"standby-{queue}-queue"
            arguments = {
                'x-dead-letter-exchange': 'orthanc-exchange',
                'x-dead-letter-routing-key': routing_key
            }

        queue_to_count = channel.queue_declare(queue=queue_name, durable=True, arguments=arguments)
        connection.close()
        return queue_to_count.method.message_count

    def purge_all_queues(self):
        pika_conn_params = self.get_rabbitmq_connection_params()
        connection = pika.BlockingConnection(pika_conn_params)
        channel = connection.channel()

        # we do nothing in catch because the queue could be not existing which is not a problem
        try:
            channel.queue_purge(queue='to-delete-queue')
        except:
            pass
        try:
            channel.queue_purge(queue='to-forward-queue')
        except:
            pass
        try:
            channel.queue_purge(queue='standby-delete-queue')
        except:
            pass
        try:
            channel.queue_purge(queue='standby-forward-queue')
        except:
            pass

        connection.close()

    def get_number_of_running_containers(self):
        bash_cmd = "docker ps | tail -n +2 | wc -l"
        running_containers = int(subprocess.check_output(bash_cmd, shell=True))
        return running_containers

    def test_forward_and_delete_instance(self):

        # clean up
        self.oa.delete_all_content()
        self.ob.delete_all_content()
        self.purge_all_queues()

        # upload an instance
        self.oa.upload_file(here / "stimuli/CT_small.dcm")

        # let's configure the replicator
        broker_connection_parameters = self.get_rabbitmq_connection_params()
        replicator = OrthancReplicator(
            source=self.oa,
            destination=self.ob,
            broker_params=broker_connection_parameters
        )

        replicator.execute()

        # wait until the instance has been forwarded to the destination
        helpers.wait_until(lambda: len(self.ob.studies.get_all_ids()) == 1, 5)

        # Let's check that there is now an instance in the destination
        self.assertEqual(len(self.oa.instances.get_all_ids()), len(self.ob.instances.get_all_ids()))

        # Let's remove the instance from the source
        self.oa.delete_all_content()

        # and check that is has been deleted from the destination
        helpers.wait_until(lambda: len(self.ob.studies.get_all_ids()) == 1, 5)
        self.assertEqual(len(self.oa.instances.get_all_ids()), len(self.ob.instances.get_all_ids()))

        replicator.stop()

    def test_retry_if_upload_fails(self):
        self.oa.delete_all_content()
        self.ob.delete_all_content()
        self.purge_all_queues()

        broker_connection_parameters = self.get_rabbitmq_connection_params()
        replicator = OrthancReplicator(
            source=self.oa,
            destination=self.ob,
            broker_params=broker_connection_parameters
        )

        replicator.execute()

        # let's inhibit the destination, so that the replicator won't be able to forward the instance we will upload
        with open(here / "docker-setup-replicator/inhibit.lua", 'rb') as f:
            lua_script = f.read()
        self.ob.execute_lua_script(lua_script)

        self.oa.upload_file(here / "stimuli/CT_small.dcm")

        # let's check until the message is in the standby queue
        helpers.wait_until(lambda: self.get_queue_length("forward", True) == 1, 5)

        # let's uninhibit the destination

        with open(here / "docker-setup-replicator/uninhibit.lua", 'rb') as f:
            lua_script = f.read()
        self.ob.execute_lua_script(lua_script)

        # Let's check that there is now an instance in the destination
        helpers.wait_until(lambda: len(self.ob.studies.get_all_ids()) == 1, 12)
        self.assertEqual(len(self.oa.instances.get_all_ids()), len(self.ob.instances.get_all_ids()))

        replicator.stop()

    def test_already_deleted_instance(self):
        self.oa.delete_all_content()
        self.ob.delete_all_content()
        self.purge_all_queues()

        broker_connection_parameters = self.get_rabbitmq_connection_params()
        replicator = OrthancReplicator(
            source=self.oa,
            destination=self.ob,
            broker_params=broker_connection_parameters
        )

        replicator.execute()

        # let's upload an instance in the source
        self.oa.upload_file(here / "stimuli/CT_small.dcm")

        # wait until it has been forwarded to the destination
        helpers.wait_until(lambda: len(self.ob.studies.get_all_ids()) == 1, 5)

        # remove it from the destination...
        self.ob.delete_all_content()

        # ...and from the source
        self.oa.delete_all_content()

        # the Replicator shouldn't retry the deletion from the destination,
        # so let's check that the queues are empty
        helpers.wait_until(lambda: self.get_queue_length("delete", False) == 0, 5)
        helpers.wait_until(lambda: self.get_queue_length("delete", True) == 0, 5)

        replicator.stop()

if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    unittest.main()




import typing
import time
import glob
import os
from .time_out import TimeOut
import threading
import logging

logger = logging.getLogger(__name__)


class OldFilesDeleter:
    """
    removes files that are older than a certain age (usefull, i.e, to remove worklists that are too old
    """

    def __init__(self, folder_to_monitor: str, timeout: float = 24*3600.0, filter: str = "*", execution_interval: float = 1*3600.0, recursive: bool = True):
        """
        :param folder_to_monitor: the folder to monitor
        :param timeout: the age (in seconds) after which the file shall be deleted
        :param filter: a filter to filter the files to be monitored by the Deleter
        :param execution_interval: the delay between two executions
        :param recursive: explore subfolders too
        """
        self._folder_to_monitor = folder_to_monitor
        self._timeout = timeout
        self._filter = filter
        self._execution_interval = execution_interval
        self._recursive = recursive
        self._thread = None
        self._execution_count = 0 # mainly used in unit tests

    def execute_once(self):
        self._execution_count += 1

        deleted_file_counter = 0
        oldest_time = time.time() - self._timeout

        glob_filter = os.path.join(self._folder_to_monitor, self._filter)
        if self._recursive:
            glob_filter = os.path.join(self._folder_to_monitor, '**/', self._filter)

        for file_path in glob.glob(glob_filter, recursive = self._recursive):
            last_modification_time = os.path.getmtime(file_path)
            if last_modification_time < oldest_time:
                logger.debug("deleting {file_path}".format(file_path = file_path))
                os.unlink(file_path)
                deleted_file_counter += 1

        logger.info("deleted {n} old file(s)".format(n = deleted_file_counter))

    def execute(self):
        self._is_running = True
        timeout = TimeOut(self._execution_interval)

        # execute once at startup and then every X
        self.execute_once()

        while self._is_running:
            timeout.wait_until_expired()
            self.execute_once()
            timeout.reset()

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()

    def start(self):
        logger.info("Starting old files deleter ({folder})".format(folder = self._folder_to_monitor))

        # create monitoring thread
        self._thread = threading.Thread(
            target = self.execute,
            name = 'OldFilesDeleter Thread'
        )
        self._thread.start()

    def stop(self):
        logger.info("Stopping old files deleter ({folder})".format(folder = self._folder_to_monitor))
        self._is_running = False
        self._thread.join()


import queue
import sys
import threading
import time
import os
import logging
import argparse
import datetime
import multiprocessing
from orthanc_api_client import helpers
from .helpers.scheduler import Scheduler
from .dicom_migrator import DicomMigrator, Message

from orthanc_api_client import OrthancApiClient
logger = logging.getLogger(__name__)

class PacsMigrator(DicomMigrator):
    """
    Uses the DicomMigrator to migrate a range of dates.
    """

    def __init__(self,
                 api_client: OrthancApiClient,
                 from_study_date: datetime.date,        # Start date
                 to_study_date: datetime.date,          # End date
                 source_modality: str = None,           # Source modality as configured in Orthanc (alias)
                 max_cfind_study_count: int = None,     # Known maximum amount of studies retrievable from the source modality at once
                 destination_modality: str = None,      # Destination modality as configured in Orthanc (alias)
                 destination_aet: str = None,           # Destination AET
                 delete_from_source: bool = False,      # once the data has been migrated, delete it from source (only vali
                 scheduler: Scheduler = None,
                 worker_threads_count: int = multiprocessing.cpu_count() - 1,  # by default, use all CPUs but one for compression
                 exit_on_error: bool = False
                 ):

        super().__init__(
            api_client=api_client,
            source_modality=source_modality,
            max_cfind_study_count=max_cfind_study_count,
            destination_modality=destination_modality,
            destination_aet=destination_aet,
            delete_from_source=delete_from_source,
            scheduler=scheduler,
            worker_threads_count=worker_threads_count,
            exit_on_error=exit_on_error
        )

        self._from_study_date = from_study_date
        self._to_study_date = to_study_date

    def execute(self):
        super().execute()

        logger.info("From Date: " + str(self._from_study_date))
        logger.info("To Date  : " + str(self._to_study_date))

        if self._from_study_date < self._to_study_date:
            # ex 20220101 -> 20220131
            inc_date = 1
            to_date = self._to_study_date + datetime.timedelta(days=1)
        else:
            # ex 20220131 -> 20220101
            inc_date = -1
            to_date = self._to_study_date - datetime.timedelta(days=1)

        current_date = self._from_study_date

        while current_date != to_date:
            logger.info("Processing date {date}".format(date=str(current_date)))

            query = self._dicom_tags_to_query
            query["StudyDate"] = helpers.to_dicom_date(current_date)

            if self.source_is_orthanc:
                logger.info("Querying Orthanc")
                local_studies = self._api_client.studies.find(query=query)

                logger.info(f"Found {len(local_studies)} studies")

                for study in local_studies:
                    self.push_message(Message(orthanc_id=study.orthanc_id))
            else:
                logger.info(f"Querying remote modality {self._source_modality}")

                retry_count = 0
                while retry_count < 5:
                    try:
                        remote_modality_studies = self._api_client.modalities.query_studies(
                            from_modality=self._source_modality,
                            query=query
                        )
                        break

                    except Exception as ex:
                        retry_count += 1
                        if retry_count == 5:
                            logger.error("Could not query the modality (retried 5 times), aborting")
                            if self._exit_on_error:
                                logger.info("exiting due to an error...")
                                self.stop_threads()
                                sys.exit(1)
                            return

                logger.info(f"Found {len(remote_modality_studies)} studies")

                if self._max_cfind_study_count and len(remote_modality_studies) == self._max_cfind_study_count:
                    logger.error(f"Too many studies in a single request: {len(remote_modality_studies)}, you'll probably miss some studies")
                    if self._exit_on_error:
                        logger.info("exiting due to an error...")
                        self.stop_threads()
                        sys.exit(1)

                for study in remote_modality_studies:
                    self.push_message(Message(dicom_id=study.dicom_id))

            current_date += datetime.timedelta(days=inc_date)

        self.stop_threads()

        logger.info("--------------------------------------------------------------------")
        logger.info("Migration completed")


# examples:
# migrator as source: MIGRATOR --> DESTINATION:  ! use the --delete_from_source option with care !!!
# python orthanc_tools/pacs_migrator.py --url=http://localhost:8042 --user=user --password=pwd --destination_modality=orthanc-debug --from_study_date=20000101 --to_study_date=20191231 --run_only_at_night_and_weekend --night_start_hour=18 --night_end_hour=6
#
#
# migrator aside from source and destination:
# SOURCE --> DESTINATION
#   |
# MIGRATOR
# python orthanc_tools/pacs_migrator.py --url=http://localhost:8044 --user=user --password=pwd --source_modality=service --destination_aet=ORTHANC --from_study_date=20000101 --to_study_date=20191231 --run_only_at_night_and_weekend --night_start_hour=18 --night_end_hour=6
#
#
# migrator as the destination: SOURCE --> MIGRATOR:
# python orthanc_tools/pacs_migrator.py --url=http://localhost:8044 --user=user --password=pwd --source_modality=service --destination_aet=ORTHANC-DEBUG --from_study_date=20000101 --to_study_date=20191231 --run_only_at_night_and_weekend --night_start_hour=18 --night_end_hour=6
#



if __name__ == '__main__':
    level = logging.INFO

    if os.environ.get('VERBOSE_ENABLED'):
        level = logging.DEBUG

    logging.basicConfig(level=level, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    parser = argparse.ArgumentParser(description='Migrate the content of a remote modality to another modality')
    parser.add_argument('--url', type=str, default=None, help='Orthanc url (migrator)')
    parser.add_argument('--user', type=str, default=None, help='Orthanc user name')
    parser.add_argument('--password', type=str, default=None, help='Orthanc password')
    parser.add_argument('--destination_modality', type=str, default=None, help='Destination modality (alias)')
    parser.add_argument('--destination_aet', type=str, default=None, help='Destination AET')
    parser.add_argument('--source_modality', type=str, default=None, help='Source modality (alias)')
    parser.add_argument('--from_study_date', type=str, required='FROM_STUDY_DATE' not in os.environ, help='From Study Date (format 20190225)')
    parser.add_argument('--to_study_date', type=str, required='TO_STUDY_DATE' not in os.environ, help='To Study Date (format 20190225)')
    parser.add_argument('--delete_from_source', default=False, action='store_true', help='delete data from source (only if source is an Orthanc)')
    parser.add_argument('--worker_threads_count', type=int, default=1, help='Worker threads count')
    parser.add_argument('--exit_on_error', default=False, action='store_true', help='if True, the script will exit in case of error')

    Scheduler.add_parser_arguments(parser)

    args = parser.parse_args()

    url = os.environ.get("ORTHANC_URL", args.url)
    user = os.environ.get("ORTHANC_USER", args.user)
    pwd = os.environ.get("ORTHANC_PWD", args.password)
    destination_modality = os.environ.get("DESTINATION_MODALITY", args.destination_modality)
    destination_aet = os.environ.get("DESTINATION_AET", args.destination_aet)
    source_modality = os.environ.get("SOURCE_MODALITY", args.source_modality)
    from_study_date = helpers.from_dicom_date(os.environ.get("FROM_STUDY_DATE", args.from_study_date))
    to_study_date = helpers.from_dicom_date(os.environ.get("TO_STUDY_DATE", args.to_study_date))
    worker_threads_count = int(os.environ.get("WORKER_THREADS_COUNT", str(args.worker_threads_count)))

    scheduler = Scheduler.create_from_args_and_env_var(args)

    if os.environ.get("DELETE_FROM_SOURCE", None) is not None:
        delete_from_source = os.environ.get("DELETE_FROM_SOURCE") == "true"
    else:
        delete_from_source = args.delete_from_source

    if os.environ.get("EXIT_ON_ERROR", None) is not None:
        exit_on_error = os.environ.get("EXIT_ON_ERROR") == "true"
    else:
        exit_on_error = args.exit_on_error

    migrator = PacsMigrator(
        api_client=OrthancApiClient(url, user=user, pwd=pwd),
        from_study_date=from_study_date,
        to_study_date=to_study_date,
        destination_modality=destination_modality,
        destination_aet=destination_aet,
        source_modality=source_modality,
        delete_from_source=delete_from_source,
        scheduler=scheduler,
        worker_threads_count=worker_threads_count,
        exit_on_error=exit_on_error
    )

    migrator.execute()



import queue
import threading
import time
import os
import logging
import argparse
import datetime
import random
import pydicom
import uuid

from orthanc_api_client import OrthancApiClient
from orthanc_api_client import helpers
logger = logging.getLogger(__name__)


male_first_names = ['Adam', 'Adrian', 'Alan', 'Alexander', 'Andrew', 'Anthony', 'Austin', 'Benjamin', 'Blake', 'Boris', 'Brandon', 'Brian', 'Cameron', 'Carl', 'Charles', 'Christian', 'Christopher',
    'Colin', 'Connor', 'Dan', 'David', 'Dominic', 'Dylan', 'Edward', 'Eric', 'Evan', 'Frank', 'Gavin', 'Gordon', 'Harry', 'Ian', 'Isaac', 'Jack', 'Jacob', 'Jake', 'James', 'Jason',
    'Joe', 'John', 'Jonathan', 'Joseph', 'Joshua', 'Julian', 'Justin', 'Keith', 'Kevin', 'Leonard', 'Liam', 'Lucas', 'Luke', 'Matt', 'Max', 'Michael', 'Nathan', 'Neil', 'Nicholas',
    'Oliver', 'Owen', 'Paul', 'Peter', 'Phil', 'Piers', 'Richard', 'Robert', 'Ryan', 'Sam', 'Sean', 'Sebastian', 'Simon', 'Stephen', 'Steven', 'Stewart', 'Thomas', 'Tim', 'Trevor',
    'Victor', 'Warren']

female_first_names = ['Abigail', 'Alexandra', 'Alison', 'Amanda', 'Amelia', 'Amy', 'Andrea', 'Angela', 'Anna', 'Anne', 'Audrey', 'Ava', 'Bella', 'Bernadette', 'Carol', 'Caroline', 'Carolyn',
    'Chloe', 'Claire', 'Deirdre', 'Diana', 'Diane', 'Donna', 'Dorothy', 'Elizabeth', 'Ella', 'Emily', 'Emma', 'Faith', 'Felicity', 'Fiona', 'Gabrielle', 'Grace', 'Hannah',
    'Heather', 'Irene', 'Jan', 'Jane', 'Jasmine', 'Jennifer', 'Jessica', 'Joan', 'Joanne', 'Julia', 'Karen', 'Katherine', 'Kimberly', 'Kylie', 'Lauren', 'Leah', 'Lillian', 'Lily',
    'Lisa', 'Madeleine', 'Maria', 'Mary', 'Megan', 'Melanie', 'Michelle', 'Molly', 'Natalie', 'Nicola', 'Olivia', 'Penelope', 'Pippa', 'Rachel', 'Rebecca', 'Rose', 'Ruth', 'Sally',
    'Samantha', 'Sarah', 'Sonia', 'Sophie', 'Stephanie', 'Sue', 'Theresa', 'Tracey', 'Una', 'Vanessa', 'Victoria', 'Virginia', 'Wanda', 'Wendy', 'Yvonne', 'Zoe']

last_names = ['Abraham', 'Allan', 'Alsop', 'Anderson', 'Arnold', 'Avery', 'Bailey', 'Baker', 'Ball', 'Bell', 'Berry', 'Black', 'Blake', 'Bond', 'Bower', 'Brown', 'Buckland', 'Burgess', 'Butler',
    'Cameron', 'Campbell', 'Carr', 'Chapman', 'Churchill', 'Clark', 'Clarkson', 'Coleman', 'Cornish', 'Davidson', 'Davies', 'Dickens', 'Dowd', 'Duncan', 'Dyer', 'Edmunds', 'Ellison',
    'Ferguson', 'Fisher', 'Forsyth', 'Fraser', 'Gibson', 'Gill', 'Glover', 'Graham', 'Grant', 'Gray', 'Greene', 'Hamilton', 'Hardacre', 'Harris', 'Hart', 'Hemmings', 'Henderson', 'Hill',
    'Hodges', 'Howard', 'Hudson', 'Hughes', 'Hunter', 'Ince', 'Jackson', 'James', 'Johnston', 'Jones', 'Kelly', 'Kerr', 'King', 'Knox', 'Lambert', 'Langdon', 'Lawrence', 'Lee', 'Lewis',
    'Lyman', 'MacDonald', 'Mackay', 'Mackenzie', 'MacLeod', 'Manning', 'Marshall', 'Martin', 'Mathis', 'May', 'McDonald', 'McLean', 'McGrath', 'Metcalfe', 'Miller', 'Mills', 'Mitchell',
    'Morgan', 'Morrison', 'Murray', 'Nash', 'Newman', 'Nolan', 'North', 'Ogden', 'Oliver', 'Paige', 'Parr', 'Parsons', 'Paterson', 'Payne', 'Peake', 'Peters', 'Piper', 'Poole', 'Powell',
    'Pullman', 'Quinn', 'Rampling', 'Randall', 'Rees', 'Reid', 'Roberts', 'Robertson', 'Ross', 'Russell', 'Rutherford', 'Sanderson', 'Scott', 'Sharp', 'Short', 'Simpson', 'Skinner',
    'Slater', 'Smith', 'Springer', 'Stewart', 'Sutherland', 'Taylor', 'Terry', 'Thomson', 'Tucker', 'Turner', 'Underwood', 'Vance', 'Vaughan', 'Walker', 'Wallace', 'Walsh', 'Watson',
    'Welch', 'White', 'Wilkins', 'Wilson', 'Wright', 'Young']


labels = ['Label1', "Label2", "Label3", "Label4", "Label5", "Label6", "Label7", "Label8", "Label9"]

class OrthancTestDbPopulator:
    """
    Populates an Orthanc with a test DB
    """

    def __init__(self,
                 api_client: OrthancApiClient,
                 studies_count: int,
                 series_count: int = None,                  # to force the number of series in a study
                 instances_count: int = None,               # to force the number of instances in a study
                 random_seed: int = None,                   # to make the generation repeatable
                 from_study_date: datetime.date = datetime.date(2000, 1, 1),     # StudyDate for generated studies
                 to_study_date: datetime.date = datetime.date(2022, 4, 21)        # StudyDate for generated studies
                 ):

        self._api_client = api_client
        self._studies_count = studies_count
        self._series_count = series_count
        self._instances_count = instances_count
        self._random_seed = random_seed
        self._patient_counter = 1
        self._from_study_date = from_study_date
        self._to_study_date = to_study_date

    def generate_patient_tags(self, tags: object) -> object:
        # generate a pseudo patient
        gender = random.choice(['M', 'F'])
        if gender == 'F':
            first_name = random.choice(female_first_names)
        else:
            first_name = random.choice(male_first_names)
        last_name = random.choice(last_names)

        tags["PatientName"] = '{0}^{1}'.format(first_name.upper(), last_name.upper())
        tags["PatientBirthDate"] = helpers.get_random_dicom_date(date_from=datetime.date(1920, 1, 1))
        tags["PatientID"] = "ID-{f}-{l}-{c:08}".format(f=first_name[:3].upper(), l=last_name[:3].upper(), c=self._patient_counter)
        tags["PatientSex"] = gender

        self._patient_counter += 1
        logger.info("created patient {id} - {name}".format(id=tags["PatientID"], name=tags["PatientName"]))

        return tags


    def generate_study_tags(self, tags: object, study_counter: int) -> object:
        tags["StudyInstanceUID"] = pydicom.uid.generate_uid(entropy_srcs=[str(study_counter), str(random.randint(0, 1000000))])
        tags["StudyDescription"] = f"Study # {study_counter:08}"
        tags["StudyDate"] = helpers.get_random_dicom_date(date_from=self._from_study_date, date_to=self._to_study_date)

        return tags

    def generate_series_tags(self, tags: object, series_counter: int, study_counter: int) -> object:
        tags["SeriesInstanceUID"] = pydicom.uid.generate_uid(entropy_srcs=[str(series_counter), str(study_counter), str(random.randint(0, 1000000))])
        tags["FrameOfReferenceUID"] = pydicom.uid.generate_uid(entropy_srcs=[str(series_counter), str(study_counter), str(random.randint(1000000, 2000000))])
        tags["SeriesDescription"] = f"Series # {series_counter:02}"
        tags["SeriesDate"] = tags["StudyDate"]
        tags["Modality"] = random.choice(["MR", "CT", "CR", "DX", "CR", "DX", "CR", "DX", "CR", "DX", "CR", "DX"])  #we want more CR and DX !

        return tags

    def generate_instance_tags(self, tags: object, instance_counter: int, series_counter: int, study_counter: int) -> object:
        tags["SOPInstanceUID"] = pydicom.uid.generate_uid(entropy_srcs=[str(instance_counter), str(series_counter), str(study_counter), str(random.randint(0, 1000000))])
        tags["InstanceNumber"] = instance_counter + 1
        tags["ContentDate"] = tags["StudyDate"]
        tags["AcquisitionDate"] = tags["StudyDate"]

        return tags

    def execute(self):
        random.seed(self._random_seed)

        tags = {}
        tags = self.generate_patient_tags(tags)

        for study_counter in range(0, self._studies_count):
            if random.randint(0, 10) > 7:  # change patients every now and then
                tags = self.generate_patient_tags(tags)

            tags = self.generate_study_tags(tags, study_counter)

            logger.info(f"-created study {tags['StudyDescription']}")

            if self._series_count is not None:
                series_count = self._series_count
            else:
                series_count = random.randint(1, 6)

            for series_counter in range(0, series_count):
                tags = self.generate_series_tags(tags, series_counter, study_counter)

                if tags["Modality"] in ["MR", "CT"]:
                    if self._instances_count is not None:
                        instances_count = self._instances_count
                    else:
                        instances_count = random.randint(50, 150)
                else:
                    instances_count = 1

                logger.info(f"--created series {tags['Modality']} with {instances_count} instances")

                for instance_counter in range(0, instances_count):
                    tags = self.generate_instance_tags(tags, instance_counter, series_counter, study_counter)

                    dicom = helpers.generate_test_dicom_file(width=2, height=2, tags=tags)
                    instance_id = self._api_client.upload(buffer=dicom)[0]

            study_id = self._api_client.instances.get_parent_study_id(instance_id)
            for labels_count in range(0, random.randint(0, 3)):
                label = labels[random.randint(0, len(labels) - 1)]
                self._api_client.studies.add_label(study_id, label)

# examples:
# python orthanc_tools/orthanc_test_db_populator.py --url=http://192.168.0.10:8042 --user=user --password=pwd --studies=50 --seed=42

if __name__ == '__main__':
    level = logging.INFO

    if os.environ.get('VERBOSE_ENABLED'):
        level = logging.DEBUG

    logging.basicConfig(level=level, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    parser = argparse.ArgumentParser(description='Populates an Orthanc with a test DB')
    parser.add_argument('--url', type=str, default=None, help='Orthanc url')
    parser.add_argument('--user', type=str, default=None, help='Orthanc user name')
    parser.add_argument('--password', type=str, default=None, help='Orthanc password')
    parser.add_argument('--studies', type=int, default=100, help='Number of studies to push')
    parser.add_argument('--series', type=int, default=None, help='Number of series in each study')
    parser.add_argument('--instances', type=int, default=None, help='Number of instances in each series')
    parser.add_argument('--seed', type=int, default=42, help='Random seed (to make generation repeatable)')
    args = parser.parse_args()

    url = os.environ.get("ORTHANC_URL", args.url)
    user = os.environ.get("ORTHANC_USER", args.user)
    pwd = os.environ.get("ORTHANC_PWD", args.password)

    populator = OrthancTestDbPopulator(
        api_client=OrthancApiClient(url, user=user, pwd=pwd),
        studies_count=args.studies,
        series_count=args.series,
        instances_count=args.instances,
        random_seed=args.seed
    )

    populator.execute()




import typing
import re

class Hl7MessageValidator:
    """
    This class validates the structure of HL7 message (it actually only checks the start/end chars and line splits)
    and returns a string with the real message.
    """

    def __init__(self, encoding: str = 'iso-8859-1', sb: bytes = b'\x0b', eb: bytes = b'\x1c', cr: bytes = b'\r'):
        self.encoding = encoding
        self.sb = sb
        self.eb = eb
        self.cr = cr
        self.validator = re.compile(self.sb.decode() + "(([^\r]+\r)*([^\r]+\r?))" + self.eb.decode() + self.cr.decode())
        self.validator_without_delimiters = re.compile("(([^\r]+\r)*([^\r]+\r?))")

    def validate(self, source: typing.Union[str, bytes], strict: bool = True) -> typing.Optional[str]:
        """
        validates the structure of HL7 message (mainly the delimiters) and returns a string with the real message in which the delimiters have been removed (the segment separator in the returned string is \r)
        :param source: a bytes array or str with the message
        :param strict: if False, it's not mandatory that the delimiters are present (i.e., whne reading the HL7 message from a file)
        :return:
        """
        if isinstance(source, str):
            bytes_message = source.encode()
        else:
            bytes_message = source

        # replace the \r\n by \r
        bytes_message = bytes_message.replace(b'\n', b'\r')
        bytes_message = bytes_message.replace(b'\r\r', b'\r')

        # first try to match the message including the delimiters
        matched = self.validator.match(bytes_message.decode(self.encoding))
        if matched is not None:
            return matched.groups()[0]

        if not strict:
            # then try to match the message without the delimiters (i.e if it comes from a file, it will not contain them)
            matched = self.validator_without_delimiters.match(bytes_message.decode(self.encoding))
            if matched is not None:
                return matched.groups()[0]

        return None



'''
# Summary
This script will clean the Orthanc by deleting the oldest studies according to the labels applied on them.

# How it works
The script will run every day at specified time.
It will read a configuration file defining what should be done:

LABEL1,6
LABEL2,12

With that sample, all studies with the LABEL1 and older than 6 weeks will be deleted
all studies with the LABEL2 and older than 12 weeks will be deleted.

Note: studies are deleted only if they were uploaded/modified in Orthanc before the retention period
'''

import datetime

import schedule
import time
import argparse
import logging
from typing import List
import os
from orthanc_api_client import OrthancApiClient, helpers
import csv

logger = logging.getLogger(__name__)

class LabelRule:
    def __init__(self,
                 label_name: str,
                 retention_duration: int    # unit: week
                 ):
        self.label_name = label_name
        self.retention_duration = retention_duration

class OrthancCleaner:

    def __init__(self,
                 api_client: OrthancApiClient,
                 execution_time: str,
                 labels_file_path: str
                 ):
        '''
        If execution_time value is 'None', the clean up will be ran only once (use case: unit tests)
        '''
        self._api_client = api_client
        self._execution_time = execution_time
        self._labels_file_path = labels_file_path

    def clean(self):
        '''
        Get information from the csv file (label and retention duration);
        Query Orthanc based on this information;
        Delete the studies found.
        '''

        # Get the information from  the csv file
        logger.info("Starting daily clean up...")
        labels_rules = self.parse_csv_file()

        logger.info(f"Rules found:")
        for rule in labels_rules:
            logger.info(f"{rule.label_name} - {rule.retention_duration} weeks")

        studies_to_delete = []

        # Query Orthanc and delete the studies for each label rule
        for label_rule in labels_rules:

            # Let's compute the date
            limit_date = self.compute_limit_date(label_rule.retention_duration)

            # Query Orthanc based on the date and the label
            studies_to_delete_by_study_date = self._api_client.studies.find(
                query={'StudyDate': f'-{helpers.to_dicom_date(limit_date)}'},
                labels=[label_rule.label_name]
                )

            # Filter the old studies which were recently stored in Orthanc
            for s in studies_to_delete_by_study_date:
                if limit_date > s.last_update.date():
                    studies_to_delete.append(s)

        # Delete the found studies
        for s in studies_to_delete:
            try:
                self._api_client.studies.delete(s.orthanc_id)
                logger.info(f"Deleting study {s.dicom_id} from {s.main_dicom_tags.get('StudyDate')}...")
            except Exception as ex:
                logger.error(f"ERROR: {str(ex)}")

        logger.info("Clean up done!")


    def compute_limit_date(self, number_of_weeks) -> datetime.date:
        limit_date = datetime.date.today() - datetime.timedelta(weeks=number_of_weeks)
        return limit_date

    def parse_csv_file(self) -> List[LabelRule]:
        '''
        Read the csv file, extract info and return a list of 'LabelRule' objects
        '''

        labels_rules = []
        with open(self._labels_file_path, 'r') as csv_file:
            reader = csv.reader(csv_file)

            for row in reader:
                labels_rules.append(LabelRule(row[0], int(row[1])))
        return labels_rules


    def execute(self):
        logger.info("----- Initializing Orthanc Cleaner...")
        if self._execution_time is None:
            # unit test case
            self.clean()
        else:
            # regular (prod) case
            schedule.every().day.at(self._execution_time).do(self.clean)
            while True:
                schedule.run_pending()
                time.sleep(1)


# example:
# python orthanc_tools/orthanc_cleaner.py --orthanc_url=http://192.168.0.10:8042 --orthanc_user=user --orthanc_pwd=pwd --execution_time=23:30 --labels_file_path=./tests/stimuli/labels.csv

if __name__ == '__main__':
    level = logging.INFO

    if os.environ.get('VERBOSE_ENABLED'):
        level = logging.DEBUG

    logging.basicConfig(level=level, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    parser = argparse.ArgumentParser(description='Clean the Orthanc by deleting the oldest studies according to the labels applied on them.')
    parser.add_argument('--orthanc_url', type=str, default=None, help='Orthanc source url')
    parser.add_argument('--orthanc_user', type=str, default=None, help='Orthanc source user name')
    parser.add_argument('--orthanc_pwd', type=str, default=None, help='Orthanc source password')
    parser.add_argument('--execution_time', type=str, default='2:30', help='Time for script execution (format: 23:30 or 23:30:14).')
    parser.add_argument('--labels_file_path', type=str, default=None, help='Path of the file containing the labels to handle and retention duration')

    args = parser.parse_args()

    orthanc_url = os.environ.get("ORTHANC_URL", args.orthanc_url)
    orthanc_user = os.environ.get("ORTHANC_USER", args.orthanc_user)
    orthanc_pwd = os.environ.get("ORTHANC_PWD", args.orthanc_pwd)
    execution_time = os.environ.get("EXECUTION_TIME", args.execution_time)
    labels_file_path = os.environ.get("LABELS_FILE_PATH", args.labels_file_path)

    cleaner = OrthancCleaner(
        api_client=OrthancApiClient(orthanc_url, user=orthanc_user, pwd=orthanc_pwd),
        execution_time=execution_time,
        labels_file_path=labels_file_path
    )

    cleaner.execute()






import os, argparse, sys, time
from orthanc_tools import MLLPServer, OldFilesDeleter
from orthanc_tools import Hl7WorklistParser, DicomWorklistBuilder, Hl7OrmWorklistMsgHandler
from orthanc_tools import Hl7ReportParser, ReportSeriesBuilder, Hl7OruReportMsgHandler
from orthanc_api_client import OrthancApiClient

import logging

# this starts an hl7 server
if __name__ == "__main__":


    if os.environ.get('VERBOSE_ENABLED'):
        level = logging.DEBUG

    logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    # build command line parser
    parser = argparse.ArgumentParser(description = 'Create a HL7 server')
    parser.add_argument('-p', '--port', help = 'Port number the HL7 Server will listen to', default = 2575, type = int)
    parser.add_argument('-f', '--folder', help = 'Folder in which the HL7 Server will store the Dicom Worklist files (.wl).  This folder shall be the one Orthanc uses as its WorklistDatabase', default = 'worklists')
    parser.add_argument('-e', '--encoding', help = 'The encoding of HL7 messages', default = 'iso-8859-1')
    parser.add_argument('-r', '--retention', help = "Number of hours to keep the WL after it has been generated", default = 24, type = int)

    args = parser.parse_args()

    port = int(os.environ.get("PORT", str(args.port)))
    folder = os.environ.get("FOLDER", args.folder)
    encoding = os.environ.get("ENCODING", args.encoding)
    retention = int(os.environ.get("RETENTION", str(args.retention)))

    # prepare worklists handler (orm)

    orm_parser = Hl7WorklistParser()
    worklist_builder = DicomWorklistBuilder(folder = folder)
    orm_handler = Hl7OrmWorklistMsgHandler(parser=orm_parser, builder=worklist_builder)

    # configure server
    mllp_server = MLLPServer(
            host = '0.0.0.0',
            port = port,
            handlers = {
            'ORM^O01': (orm_handler.handle_orm_message,)
            }
    )

    # start server
    mllp_server.start()

    # configurer and start old_files_deleter
    worklist_cleaner = OldFilesDeleter(folder_to_monitor = folder, timeout = retention*3600.0, filter = '*.wl', execution_interval = 3600.0)
    worklist_cleaner.start()

    while True:
        time.sleep(1)

